#!/usr/bin/env python

#  
#  Copyright (C) 2010 Calum Lind, Rico Tzschichholz
# 
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
# 
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import atexit

from twisted.internet import glib2reactor # for gtk-2.0
glib2reactor.install()

try:
	from deluge.ui.client import client
	from twisted.internet import reactor
	from docky.docky import DockyItem, DockySink
	from signal import signal, SIGTERM
	from sys import exit
	from gobject import timeout_add
except ImportError, e:
	print e
	exit()


myresult = 0
ShowUploadRate = False
host="localhost"
port=58846
user=""
password=""
d = client.connect(host, port, user, password)


def on_connect_success(result):
	def on_get_config_value(value, key):
		global myresult
		myresult=value[key]
	client.core.get_session_status(["download_rate"]).addCallback(on_get_config_value,"download_rate")

def on_connect_fail(result):
	print "Connection failed!"
	print "result:", result



def bytes2ratestr(bytes):
	for factor, suffix in (
		(1024 ** 5, 'P'),
		(1024 ** 4, 'T'), 
		(1024 ** 3, 'G'), 
		(1024 ** 2, 'M'), 
		(1024 ** 1, 'K'),
		(1024 ** 0, 'B')):
		if bytes >= factor:
			break
	amount = int(bytes / factor)
	return str(amount) + suffix
	

class DockyDelugeItem(DockyItem):
	def __init__(self, path):
		DockyItem.__init__(self, path)
		
		self.timer = 0

		if not self.timer > 0:
			self.timer = timeout_add (2000, self.update_badge)

	def update_badge(self):
		rate = 0
		try:
			if ShowUploadRate:
				rate = 0
			else:	
				d.addCallback(on_connect_success)
				d.addErrback(on_connect_fail)
				global myresult
				#print myresult
				rate = round(myresult)

			if rate > 0:
				self.iface.SetBadgeText("%s" % bytes2ratestr(rate))
				#self.iface.SetText("Deluge %s" % bytes2ratestr(rate))
			else:
				self.iface.ResetBadgeText()
			return True
		except Exception, e:
			print e
			self.iface.ResetBadgeText()
			return False
	

class DockyDelugeSink(DockySink):
	def item_path_found(self, pathtoitem, item):
		if item.GetOwnsDesktopFile() and item.GetDesktopFile().endswith ("deluge.desktop"):
			self.items[pathtoitem] = DockyDelugeItem(pathtoitem)

dockysink = DockyDelugeSink()

def cleanup ():
	dockysink.dispose ()

if __name__ == "__main__":
	atexit.register (cleanup)
	signal(SIGTERM, lambda signum, stack_frame: exit(1))
	reactor.run()

